@@ -1,16 +1,12 @@
-import os
 import zipfile
 from concurrent import futures
 from io import BytesIO
-from typing import Any, MutableMapping, List, Dict
+from typing import MutableMapping, List, Dict
 
-import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import ujson
-from matplotlib.widgets import Slider, Button
-from pyquaternion import Quaternion
-from scaleapi import Task
+from scaleapi.tasks import Task, TaskType
 
 from .camera import LidarCamera
 from .image import LidarImage
@@ -21,8 +17,9 @@ from .helper import s3_smart_upload, get_signed_url
 
 UPLOAD_POOL_SIZE = 8
 
+
 class LidarScene:
-    """LidarScene object represent all frames in a scene.
+    """LidarScene object representing all frames in a scene.
 
     Scene properties:
       - cameras: List of cameras
@@ -38,9 +35,10 @@ class LidarScene:
         self.cameras: MutableMapping[LidarCamera] = pd.Series()
         self.frames: MutableMapping[LidarFrame] = pd.Series()
         self.public_url = None
+        self.scale_file_attachments = None
 
-    def get_camera(self, camera_id=None, index: int=None) -> LidarCamera:
-        """Get a camera by id (or index) and creates one if It doesn't exists
+    def get_camera(self, camera_id=None, index: int = None) -> LidarCamera:
+        """Get a camera by id (or index) or create one if it does not exist
 
         :param camera_id: The camera id
         :type camera_id: str, int
@@ -49,8 +47,11 @@ class LidarScene:
 
         :return: LidarCamera
         :rtype:  LidarCamera
+
         """
-        assert camera_id is not None or index is not None, 'id or index must be specified'
+        assert (
+            camera_id is not None or index is not None
+        ), "id or index must be specified"
 
         if camera_id is None:
             camera_id = self.cameras.index[index]
@@ -61,8 +62,8 @@ class LidarScene:
             self.cameras[camera_id] = LidarCamera(camera_id)
         return self.cameras[camera_id]
 
-    def get_frame(self, frame_id=None, index: int=None) -> LidarFrame:
-        """Get a frame by id (or index) or creates one if It doesn't exists
+    def get_frame(self, frame_id=None, index: int = None) -> LidarFrame:
+        """Get a frame by id (or index) or create one if it does not exist
 
         :param frame_id: The frame id
         :type frame_id: str, int
@@ -70,8 +71,12 @@ class LidarScene:
         :type index: int
 
         :return: LidarFrame
+        :rtype: LidarFrame
+
         """
-        assert frame_id is not None or index is not None, 'id or index must be specified'
+        assert (
+            frame_id is not None or index is not None
+        ), "id or index must be specified"
 
         if frame_id is None:
             frame_id = self.frames.index[index]
@@ -83,18 +88,21 @@ class LidarScene:
         return self.frames[frame_id]
 
     def apply_transforms(self, world_transforms: List[Transform]):
-        """Apply transformations to all the frames (the number of Transformation should be the same as the number of frames)
+        """Apply transformations to all the frames (the number of Transformation should match the number of frames)
 
         :param world_transforms: List of Transform
         :type world_transforms: list(Transform)
+
         """
-        assert len(world_transforms) != len(self.frames), 'world_transforms should have the same length as frames'
+        assert len(world_transforms) != len(
+            self.frames
+        ), "world_transforms should have the same length as frames"
 
         for idx in range(len(self.frames)):
             self.get_frame(index=idx).apply_transform(world_transforms[idx])
 
     def filter_points(self, min_intensity=None, min_intensity_percentile=None):
-        """Filter points based on their intensity
+        """Filter points based on intensity
 
         :param min_intensity: Minimun intensity allowed
         :type min_intensity: int
@@ -105,7 +113,9 @@ class LidarScene:
         for frame in self.frames:
             frame.filter_points(min_intensity, min_intensity_percentile)
 
-    def get_projected_image(self, camera_id, color_mode='intensity', frames_index=range(0,1), **kwargs):
+    def get_projected_image(
+        self, camera_id, color_mode="intensity", frames_index=range(0, 1), **kwargs
+    ):
         """Get camera_id image with projected points, (**Legacy method**)
 
         :param camera_id: Camera id/Name/Identifier
@@ -119,13 +129,18 @@ class LidarScene:
         :rtype: Image
 
         """
-        all_points = np.array([]).reshape(0,4)
+        all_points = np.array([]).reshape(0, 4)
         for idx in frames_index:
-            points = np.array(self.frames[idx].points)[:,:4]
+            points = np.array(self.frames[idx].points)[:, :4]
             print(points.shape)
             points[:, 3] = idx
             all_points = np.concatenate((all_points, points), axis=0)
-        return self.cameras[camera_id].get_projected_image(self.frames[frames_index[0]].get_image(camera_id), all_points, self.frames[frames_index[0]].transform, color_mode)
+        return self.cameras[camera_id].get_projected_image(
+            self.frames[frames_index[0]].get_image(camera_id),
+            all_points,
+            self.frames[frames_index[0]].transform,
+            color_mode,
+        )
 
     def apply_transform(self, world_transform: Transform):
         """Apply a Transformation to all the frames
@@ -139,15 +154,13 @@ class LidarScene:
             self.get_frame(index=idx).apply_transform(world_transform)
 
     def make_transforms_relative(self):
-        """Make all the frame transform relative to the first transform/frame. This will set the first transform to position (0,0,0) and heading (1,0,0,0)
-
-        """
+        """Make all the frame transform relative to the first transform/frame. This will set the first transform to position (0,0,0) and heading (1,0,0,0)"""
         offset = self.get_frame(index=0).transform.inverse
         for frame in self.frames:
             frame.transform = offset @ frame.transform
 
-    def to_dict(self, public_url: str=None) -> dict:
-        """Return a dictionary with the frames urls using the public_url as base.
+    def to_dict(self, public_url: str = None) -> dict:
+        """Return a dictionary with the frame urls using the public_url as base.
 
         :param public_url: This url will be concatenated with the frames name, e.g.: `'%s/frame-%s.json' % (public_url, frame.id)`
         :type public_url: str
@@ -157,14 +170,24 @@ class LidarScene:
         """
         if public_url is None:
             public_url = self.public_url
-        return dict(frames=['%s/frame-%s.json' % (public_url, frame.id) for frame in self.frames])
+        return dict(
+            frames=[
+                "%s/frame-%s.json" % (public_url, frame.id) for frame in self.frames
+            ]
+        )
 
-    def s3_upload(self, bucket: str, path=None, mock_upload: float=False, use_threads: float=True):
+    def s3_upload(
+        self,
+        bucket: str,
+        path=None,
+        mock_upload: float = False,
+        use_threads: float = True,
+    ):
         """Save scene in S3
 
         :param bucket: S3 Bucket name
         :type bucket: str
-        :param path: Path where store the data
+        :param path: Path to store data
         :type key: str
         :param mock_upload: To avoid upload the data to S3  (defualt ``False``)
         :type mock_upload: float
@@ -177,31 +200,34 @@ class LidarScene:
         """
         self.public_url = f"s3://{bucket}/{path}"
 
-        print('Uploading scene to S3: %s' % self.public_url)
+        print("Uploading scene to S3: %s" % self.public_url)
         scene_dict = self.to_dict(self.public_url)
 
-        poses_csv = pd.DataFrame(self.frames.map(lambda f: list(f.transform.matrix.reshape(-1))).to_dict()).T.to_csv(
-            header=False)
+        poses_csv = pd.DataFrame(
+            self.frames.map(lambda f: list(f.transform.matrix.reshape(-1))).to_dict()
+        ).T.to_csv(header=False)
 
         if not mock_upload:
             # Upload scene json file
             s3_smart_upload(
                 bucket=bucket,
-                key=f'{path}/scene.json',
-                fileobj=BytesIO(bytes(ujson.dumps(scene_dict), encoding='utf-8')),
-                content_type='application/json'
+                key=f"{path}/scene.json",
+                fileobj=BytesIO(bytes(ujson.dumps(scene_dict), encoding="utf-8")),
+                content_type="application/json",
             )
 
             # Upload ego2world csv file
             s3_smart_upload(
                 bucket=bucket,
-                key=f'{path}/ego2world.csv',
-                fileobj=BytesIO(bytes(poses_csv, encoding='utf-8')),
-                content_type='text/plain'
+                key=f"{path}/ego2world.csv",
+                fileobj=BytesIO(bytes(poses_csv, encoding="utf-8")),
+                content_type="text/plain",
             )
 
             if use_threads:
-                with futures.ThreadPoolExecutor(max_workers=UPLOAD_POOL_SIZE) as executor:
+                with futures.ThreadPoolExecutor(
+                    max_workers=UPLOAD_POOL_SIZE
+                ) as executor:
                     future_list = [
                         executor.submit(LidarFrame.s3_upload, frame, bucket, path)
                         for frame in self.frames
@@ -216,59 +242,94 @@ class LidarScene:
 
         signed_url = get_signed_url(bucket, path)
 
-        print(f'Scene uploaded: {signed_url}')
+        print(f"Scene uploaded: {signed_url}")
         return self.public_url
 
+    def scale_file_upload(
+        self,
+        project_name: str,
+    ):
+        """Save scene in Scale file
+
+        :param project_name: File project name
+        :type project_name: str
+
+        :return: Scene file url
+        :rtype: str
+
+        """
+
+        print("Uploading scene to Scale file")
+
+        frames_attachments_urls = []
+        for frame in self.frames:
+            print(f"Uploading frame {frame.id} to Scale file")
+            frames_attachments_urls.append(frame.scale_file_upload(project_name))
+
+        self.scale_file_attachments = frames_attachments_urls
+        return self.scale_file_attachments
+
     def save_task(self, filepath: str, template=None):
-        """Save the entire scene (with frame and images) into filepath in zipfile format
+        """Save the entire scene (with frame and images) in zipfile format to local filepath
 
         :param filepath: File name and path in which the scene should be saved
         :type filepath: str
 
         """
-        print('Saving scene:', filepath)
-        with zipfile.ZipFile(filepath, mode='w') as out:
+        print("Saving scene:", filepath)
+        with zipfile.ZipFile(filepath, mode="w") as out:
             # Save task
             scene_dict = self.to_dict()
             task_dict = dict(
-                template or {},
-                attachment_type='json',
-                attachments=scene_dict['frames']
+                template or {}, attachment_type="json", attachments=scene_dict["frames"]
             )
-            out.writestr('task.json', ujson.dumps(task_dict))
+            out.writestr("task.json", ujson.dumps(task_dict))
 
             # Save frames
             for frame in self.frames:
                 # Save points
                 data = frame.to_json(self.public_url)
-                out.writestr('frame-%s.json' % frame.id, data, compress_type=zipfile.ZIP_DEFLATED)
+                out.writestr(
+                    "frame-%s.json" % frame.id, data, compress_type=zipfile.ZIP_DEFLATED
+                )
 
                 # Save frame images
                 for camera_id, image in frame.images.items():
                     if not image.image_path:
-                        assert NotImplemented('Only file-imported Images supported')
-                    out.write(image.image_path, 'image-%s-%s.jpg' % (camera_id, frame.id))
-        print('Scene saved.')
+                        assert NotImplementedError(
+                            "Only file-imported Images supported"
+                        )
+                    out.write(
+                        image.image_path, "image-%s-%s.jpg" % (camera_id, frame.id)
+                    )
+        print("Scene saved.")
 
-    def create_task(self, template: Dict = None, task_type: str = 'lidarannotation') -> Task:
-        """ Creates a Scale platform task from the configured scene
+    def create_task(
+        self, template: Dict = None, task_type: TaskType = TaskType.LidarAnnotation
+    ) -> Task:
+        """Create a Scale platform task from the configured scene
 
         :param template: Dictionary of payload for task creation (https://private-docs.scale.com/?python#parameters), attachments data will be filled automatically.
         :type template: dict
-        :param task_type: Select a Scale API endpoint top upload data to, currently supports 'lidarannotation' and 'lidartopdown'. Defaults to 'lidarannotation'.
+        :param task_type: Select a Scale API endpoint top upload data to, currently supports 'lidarannotation', 'lidarsegmentation', and 'lidartopdown'. Defaults to 'lidarannotation'.
         :type task_type: str
 
         :return: Task object with related information. Inherited from `scaleapi.Task` object.
         :rtype: Task
         """
-        if task_type == 'lidarannotation':
+        if task_type == TaskType.LidarAnnotation:
             from .task import LidarAnnotationTask
+
             return LidarAnnotationTask.from_scene(self, template)
-        elif task_type == 'lidartopdown':
+        elif task_type == TaskType.LidarTopdown:
             from .task import LidarTopDownTask
+
             return LidarTopDownTask.from_scene(self, template)
-        elif task_type == 'lidarsegmentation':
+        elif task_type == TaskType.LidarSegmentation:
             from .task import LidarSegmentationTask
-            return LidarSegmentation.from_scene(self, template)
+
+            return LidarSegmentationTask.from_scene(self, template)
         else:
-            raise NotImplemented(f'Specified task_type {task_type} is not supported')
+            raise NotImplementedError(
+                f"Specified task_type {task_type} is not supported"
+            )
